Imports System
Imports System.Collections
Imports TXTextControl
Imports Keyoti.RapidSpell


''' <summary>
''' TXHelper - glues TX and RapidSpell together.
''' </summary>
Public Class TXHelper
    Private ayt As RapidSpellAsYouType
    Private dia As RapidSpellDialog
    Private WithEvents txc As TXTextControl.TextControl
    Private dialogAdapter As TXTextControlAdapter
    Private adapters() As TXTextControlAdapter
    Private _isTextFrameActive = False
    Private defaultMenu As ContextMenuStrip

    Public ReadOnly Property IsTextFrameActive() As Boolean
        Get
            Return _isTextFrameActive
        End Get
    End Property



    ' Configures the spell checkers to work with the text control, pass a null
    ' spell checker object if not being used.
    Public Sub New(ByRef ayt As RapidSpellAsYouType, ByRef dia As RapidSpellDialog, ByRef txc As TXTextControl.TextControl, ByRef defaultMenu As ContextMenuStrip)
        Me.ayt = ayt
        Me.dia = dia
        Me.txc = txc
        Me.defaultMenu = defaultMenu

        If Not (dia Is Nothing) Then
            dialogAdapter = New TXTextControlAdapter(txc)
            dialogAdapter.WorkWithCurrentlyActiveElementOnly = True
            Me.dia.ThirdPartyTextComponentToCheck = dialogAdapter
            AddHandler dia.SpellCheckStarted, AddressOf dia_SpellCheckStarted
            AddHandler dia.SpellCheckFinished, AddressOf dia_SpellCheckFinished
        End If

        If Not (ayt Is Nothing) Then
            AddHandler Me.txc.TextFrameCreated, AddressOf txc_TextFrameCreated
            adapters = CType(getBasisAdapters().ToArray(GetType(TXTextControlAdapter)), TXTextControlAdapter())
            Me.ayt.UpdateAllTextBoxes = False
            Me.ayt.TextComponents = adapters
        End If

    End Sub 'New


    Public Sub Dispose()


        If (Not dia Is Nothing) Then
            RemoveHandler dia.SpellCheckStarted, New Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(AddressOf dia_SpellCheckStarted)
            RemoveHandler dia.SpellCheckFinished, New Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(AddressOf dia_SpellCheckFinished)
        End If

        If (Not txc Is Nothing And Not txc.IsDisposed) Then
            RemoveHandler txc.TextFrameCreated, New TextFrameEventHandler(AddressOf txc_TextFrameCreated)
            RemoveHandler txc.TextFrameActivated, New TextFrameEventHandler(AddressOf txc_TextFrameActivated)
            RemoveHandler txc.TextFrameDeactivated, New TextFrameEventHandler(AddressOf txc_TextFrameDeactivated)
        End If
    End Sub


    Function getBasisAdapters() As ArrayList
        Dim adaps As New ArrayList
        Dim adap As TXTextControlAdapter

        'add main adapter
        adap = New TXTextControlAdapter(Me.txc)
        adaps.Add(adap)
        adap.ContextMenuStripDefault = defaultMenu

        'add adapters for headers/footers
        Dim hf As HeaderFooter
        If (Not txc.HeadersAndFooters Is Nothing) Then
            For Each hf In Me.txc.HeadersAndFooters
                adap = New TXTextControlAdapter(Me.txc)
                adap.HeaderFooter = hf
                adap.ContextMenuStripDefault = defaultMenu
                adaps.Add(adap)
            Next hf
        End If
        Return adaps
    End Function 'getBasisAdapters


    Private Sub dia_SpellCheckStarted(ByVal sender As Object, ByVal e As Keyoti.RapidSpell.Event.SpellCheckEventArgs)
        dialogAdapter.IgnoreHeaderFooterDeactivationEvents = True
    End Sub 'dia_SpellCheckStarted


    Private Sub dia_SpellCheckFinished(ByVal sender As Object, ByVal e As Keyoti.RapidSpell.Event.SpellCheckEventArgs)
        dialogAdapter.IgnoreHeaderFooterDeactivationEvents = False
    End Sub 'dia_SpellCheckFinished


    'used for AYT only, this is only called when text frames come from the CLIPBOARD
    Private Sub txc_TextFrameCreated(ByVal sender As Object, ByVal e As TextFrameEventArgs)
        OnTextFrameCreated(e.TextFrame)
    End Sub 'txc_TextFrameCreated


    'used for AYT only
    ' Call when adding a TextFrame
    Public Sub OnTextFrameCreated(ByVal tf As TextFrame)
        Dim basis As New ArrayList
        Dim adap As TXTextControlAdapter

        'refresh old adapters, note we must create new adapters as old ones are retired below
        Dim a As TXTextControlAdapter
        For Each a In adapters

            adap = New TXTextControlAdapter(Me.txc)
            adap.SetWith(a)
            basis.Add(adap)
        Next a

        'text frame adapter
        adap = New TXTextControlAdapter(Me.txc)
        adap.TextFrame = tf
        basis.Add(adap)

        adapters = CType(basis.ToArray(GetType(TXTextControlAdapter)), TXTextControlAdapter())

        'retire the old ones
        ayt.TextComponents = Nothing
        'add the new ones
        ayt.TextComponents = adapters
    End Sub 'OnTextFrameCreated


    Private Sub txc_TextFrameActivated(ByVal sender As Object, ByVal e As TXTextControl.TextFrameEventArgs) Handles txc.TextFrameActivated
        Me._isTextFrameActive = True
    End Sub

    Private Sub txc_TextFrameDeactivated(ByVal sender As Object, ByVal e As TXTextControl.TextFrameEventArgs) Handles txc.TextFrameDeactivated
        Me._isTextFrameActive = False
    End Sub

    Public Sub OnDocumentOpen()
       Dim bb as ButtonBar = txc.ButtonBar
		Dim rb as RulerBar  = txc.RulerBar
		Dim vrb as RulerBar  = txc.VerticalRulerBar
		Dim sb as TXTextControl.StatusBar  = txc.StatusBar

		txc.ButtonBar = Nothing
		txc.RulerBar = Nothing
		txc.VerticalRulerBar = Nothing
		txc.StatusBar = Nothing
	   



        adapters(0).OnTextChanged(txc, EventArgs.Empty)

		txc.ButtonBar = bb
		txc.RulerBar = rb
		txc.VerticalRulerBar = vrb
		txc.StatusBar = sb
    End Sub
End Class 'TXHelper
