Imports System.Reflection
Public Class frmMain

#Region " Form Properties "
    Private FileHandler1 As FileHandler = New FileHandler(Me)
    Private m_LoadFileOnCreate As Boolean = False
    Private m_ActiveHeaderFooter As TXTextControl.HeaderFooter
    Dim helper As TXHelper

    Public Sub New()
        InitializeComponent()
        FileHandler1.RecentFilesMenuItem = RecentFilesToolStripMenuItem
        textControl1.ButtonBar = buttonBar1
        textControl1.RulerBar = rulerBar1
        textControl1.VerticalRulerBar = rulerBar2
        textControl1.StatusBar = statusBar1
    End Sub

    Private Sub frmMain_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
        Dim commands As String = Microsoft.VisualBasic.Command()
        LoadAppSettings()
        FileHandler1.TextControl = textControl1
        ' Check if program has been started with a file name as a command line parameter
        If commands = "" Then
            FileNew()
        Else
            If commands.StartsWith("""") Then commands = commands.Replace("""", "")
            If commands.EndsWith("""") Then commands = commands.Replace("""", "")
            m_LoadFileOnCreate = True
            FileHandler1.DocumentFileName = commands
        End If
        If m_LoadFileOnCreate Then
            If Not FileHandler1.FileOpen Then
                Return
            End If
            If Not (FileHandler1.DocumentFileName = "") Then
                Me.Text = FileHandler1.DocumentFileName
            End If
        End If

        Dim hfc As TXTextControl.HeaderFooterCollection
        hfc = Me.textControl1.HeadersAndFooters
        hfc.Add(TXTextControl.HeaderFooterType.Header)
        hfc.Add(TXTextControl.HeaderFooterType.Footer)

        'the second argument should be RapidSpellDialog if it is being used aswell
        helper = New TXHelper(rapidSpellAsYouType1, RapidSpellDialog1, Me.textControl1, ContextMenu_Text)


        FileHandler1.DocumentDirty = False
        UpdateSaveStatus()
    End Sub

    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles Me.FormClosing
        Dim Result As System.Windows.Forms.DialogResult
        If FileHandler1.DocumentDirty Then
            Result = MessageBox.Show("Save changes to " + Me.Text + "?", "Question", MessageBoxButtons.YesNoCancel)
            If Result = System.Windows.Forms.DialogResult.Yes Then
                FileHandler1.FileSave()
                If FileHandler1.DocumentFileName = "" Then
                    e.Cancel = True
                End If
            Else
                If Result = System.Windows.Forms.DialogResult.Cancel Then
                    e.Cancel = True
                End If
            End If
        End If
        SaveAppSettings()
    End Sub

    Public ReadOnly Property CanCopy() As Boolean
        Get
            Return textControl1.CanCopy
        End Get
    End Property

    Public ReadOnly Property CanPaste() As Boolean
        Get
            Return textControl1.CanPaste
        End Get
    End Property

    Public ReadOnly Property CanUndo() As Boolean
        Get
            Return textControl1.CanUndo
        End Get
    End Property

    Public ReadOnly Property CanRedo() As Boolean
        Get
            Return textControl1.CanRedo
        End Get
    End Property

    Public WriteOnly Property LoadFileOnCreate() As Boolean
        Set(ByVal value As Boolean)
            m_LoadFileOnCreate = value
        End Set
    End Property
#End Region

#Region " Form functions  "
    Private Sub UpdateSaveStatus()
        ' Enable or disable the save button and menu entry
        toolStrip1.Items(2).Enabled = FileHandler1.DocumentDirty
        mnuFile.DropDownItems(4).Enabled = FileHandler1.DocumentDirty
    End Sub

    Private Sub LoadAppSettings()
        ' Take over initial resizing
        Me.StartPosition = FormStartPosition.Manual

        ' Resize
        Me.Location = My.Settings.LastWindowPos
        Me.Size = My.Settings.LastWindowSize
        Me.WindowState = My.Settings.LastWindowState
    End Sub

    Private Sub SaveAppSettings()
        My.Settings.LastWindowState = Me.WindowState

        If Me.WindowState = FormWindowState.Normal Then
            My.Settings.LastWindowSize = Me.Size
            My.Settings.LastWindowPos = Me.Location
        Else
            My.Settings.LastWindowSize = Me.RestoreBounds.Size
            My.Settings.LastWindowPos = Me.RestoreBounds.Location
        End If
        My.Settings.Save()
    End Sub
#End Region

#Region " Drag and Drop "

    Private Sub frmMain_DragOver(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragOver
        If e.Data.GetDataPresent(DataFormats.FileDrop, False) = True Then
            e.Effect = DragDropEffects.All
        End If
    End Sub

    Private Sub frmMain_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        Dim files As String() = e.Data.GetData(DataFormats.FileDrop)

        Dim Result As System.Windows.Forms.DialogResult

        If FileHandler1.DocumentDirty Then
            Result = MessageBox.Show("Save changes to " + Me.Text + "?", "Question", MessageBoxButtons.YesNoCancel)
            If Result = Windows.Forms.DialogResult.Yes Then
                FileHandler1.FileSave()
            ElseIf Result = Windows.Forms.DialogResult.Cancel Then
                Return
            End If
        End If

        For Each file As String In files
            FileHandler1.DocumentFileName = file
            Me.Text = FileHandler1.DocumentFileName
            FileHandler1.FileOpen()
            ' loop through all files, if MDI is implemented
            Exit For
        Next
    End Sub

#End Region

#Region " File Menu "

    Private Sub mnuHelp_AboutTXTextControlWords_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mnuHelp_AboutTXTextControlWords.Click
        About()
    End Sub

    Private Sub mnuFile_DropDownOpening(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFile.DropDownOpening
        FileHandler1.RecentFilesMenuItem = RecentFilesToolStripMenuItem
        FileHandler1.InitRecentFiles()
    End Sub

    Private Sub mnuFile_Open_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mnuFile_Open.Click
        FileOpen()
        UpdateSaveStatus()
    End Sub

    Private Sub mnuFile_New_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mnuFile_New.Click
        FileNew()
        UpdateSaveStatus()
    End Sub

    Private Sub mnuFile_Save_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_Save.Click
        FileSave()
        UpdateSaveStatus()
    End Sub

    Private Sub mnuFile_SaveAs_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_SaveAs.Click
        Try
            FileHandler1.FileSaveAs()
            If Not (FileHandler1.DocumentFileName = "") Then
                Me.Text = FileHandler1.DocumentFileName
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
        UpdateSaveStatus()
    End Sub

    Private Sub mnuFile_Export_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_Export.Click
        Try
            FileHandler1.Export()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuFile_PageSetup_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_PageSetup.Click
        textControl1.SectionFormatDialog(0)
    End Sub

    Private Sub mnuFile_PrintPreview_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_PrintPreview.Click
        PrintPreview()
    End Sub

    Private Sub mnuFile_Print_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFile_Print.Click
        Print()
    End Sub

    Private Sub mnuFile_Exit_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mnuFile_Exit.Click
        Close()
    End Sub
#End Region

#Region " Table Menu "
    Private Sub mnuTable_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable.DropDownOpening
        Dim ThisTable As TXTextControl.Table = textControl1.Tables.GetItem
        mnuTable_Properties.Enabled = (Not (ThisTable Is Nothing))
        mnuTable_GridLines.Checked = textControl1.Tables.GridLines
        If Not (ThisTable Is Nothing) Then
            mnuTable_Split.Enabled = ThisTable.CanSplit
        Else
            mnuTable_Split.Enabled = False
        End If
    End Sub

    Private Sub mnuTable_Insert_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert.DropDownOpening
        mnuTable_Insert_Table.Enabled = textControl1.Tables.CanAdd
        Dim TableAtInputPosition As TXTextControl.Table = textControl1.Tables.GetItem
        If TableAtInputPosition Is Nothing Then
            mnuTable_Insert_ColumnToTheLeft.Enabled = False
            mnuTable_Insert_ColumnToTheRight.Enabled = False
            mnuTable_Insert_RowAbove.Enabled = False
            mnuTable_Insert_RowBelow.Enabled = False
        Else
            mnuTable_Insert_ColumnToTheLeft.Enabled = TableAtInputPosition.Columns.CanAdd
            mnuTable_Insert_ColumnToTheRight.Enabled = TableAtInputPosition.Columns.CanAdd
            mnuTable_Insert_RowAbove.Enabled = TableAtInputPosition.Rows.CanAdd
            mnuTable_Insert_RowBelow.Enabled = TableAtInputPosition.Rows.CanAdd
        End If
    End Sub

    Private Sub mnuTable_Insert_Table_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert_Table.Click
        Dim InsertTableDialog As frmInsertTable = New frmInsertTable
        InsertTableDialog.tx = textControl1
        InsertTableDialog.ShowDialog()
    End Sub

    Private Sub mnuTable_Insert_ColumnToTheLeft_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert_ColumnToTheLeft.Click
        textControl1.Tables.GetItem.Columns.Add(TXTextControl.TableAddPosition.Before)
    End Sub

    Private Sub mnuTable_Insert_ColumnToTheRight_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert_ColumnToTheRight.Click
        textControl1.Tables.GetItem.Columns.Add(TXTextControl.TableAddPosition.After)
    End Sub

    Private Sub mnuTable_Insert_RowAbove_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert_RowAbove.Click
        textControl1.Tables.GetItem.Rows.Add(TXTextControl.TableAddPosition.Before, 1)
    End Sub

    Private Sub mnuTable_Insert_RowBelow_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Insert_RowBelow.Click
        textControl1.Tables.GetItem.Rows.Add(TXTextControl.TableAddPosition.After, 1)
    End Sub

    Private Sub mnuTable_Delete_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Delete.DropDownOpening
        Dim TableAtInputPosition As TXTextControl.Table = textControl1.Tables.GetItem
        If TableAtInputPosition Is Nothing Then
            mnuTable_Delete_Table.Enabled = False
            mnuTable_Delete_Column.Enabled = False
            mnuTable_Delete_Rows.Enabled = False
        Else
            mnuTable_Delete_Table.Enabled = TableAtInputPosition.Columns.CanRemove
            mnuTable_Delete_Column.Enabled = TableAtInputPosition.Columns.CanRemove
            mnuTable_Delete_Rows.Enabled = TableAtInputPosition.Rows.CanRemove
        End If
    End Sub

    Private Sub mnuTable_Delete_Table_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Delete_Table.Click
        textControl1.Tables.Remove()
    End Sub

    Private Sub mnuTable_Delete_Column_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Delete_Column.Click
        textControl1.Tables.GetItem.Columns.Remove()
    End Sub

    Private Sub mnuTable_Delete_Rows_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Delete_Rows.Click
        textControl1.Tables.GetItem.Rows.Remove()
    End Sub

    Private Sub mnuTable_Split_Above_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Split_Above.Click
        textControl1.Tables.GetItem.Split(TXTextControl.TableAddPosition.Before)
    End Sub

    Private Sub mnuTable_Split_Below_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Split_Below.Click
        textControl1.Tables.GetItem.Split(TXTextControl.TableAddPosition.After)
    End Sub

    Private Sub mnuTable_Select_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Select.DropDownOpening
        Dim TableAtInputPosition As TXTextControl.Table = Nothing
        Dim RowAtInputPosition As TXTextControl.TableRow = Nothing
        Dim CellAtInputPosition As TXTextControl.TableCell = Nothing
        TableAtInputPosition = textControl1.Tables.GetItem
        If Not (TableAtInputPosition Is Nothing) Then
            RowAtInputPosition = TableAtInputPosition.Rows.GetItem
            CellAtInputPosition = TableAtInputPosition.Cells.GetItem
        End If
        mnuTable_Select_Table.Enabled = (Not (TableAtInputPosition Is Nothing))
        mnuTable_Select_Row.Enabled = (Not (RowAtInputPosition Is Nothing))
        mnuTable_Select_Cell.Enabled = (Not (CellAtInputPosition Is Nothing))
    End Sub

    Private Sub mnuTable_Select_Table_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Select_Table.Click
        textControl1.Tables.GetItem.Select()
    End Sub

    Private Sub mnuTable_Select_Row_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Select_Row.Click
        textControl1.Tables.GetItem.Rows.GetItem.Select()
    End Sub

    Private Sub mnuTable_Select_Cell_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Select_Cell.Click
        textControl1.Tables.GetItem.Cells.GetItem.Select()
    End Sub

    Private Sub mnuTable_GridLines_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_GridLines.Click
        textControl1.Tables.GridLines = Not textControl1.Tables.GridLines
    End Sub

    Private Sub mnuTable_Properties_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuTable_Properties.Click
        textControl1.TableFormatDialog()
    End Sub
#End Region

#Region " Options Menu "

    Private Sub mnuOptions_HTML_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuOptions_HTML.Click
        Dim HTMLOptionsDialog As frmHTMLOptions = New frmHTMLOptions
        HTMLOptionsDialog.FileHandler1 = FileHandler1
        HTMLOptionsDialog.ShowDialog()
    End Sub

    Private Sub mnuOptions_PDF_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuOptions_PDF.Click
        Dim PDFOptionsDialog As frmPDFOptions = New frmPDFOptions
        PDFOptionsDialog.FileHandler1 = FileHandler1
        PDFOptionsDialog.ShowDialog()
    End Sub
#End Region

#Region " Format Menu "

    Private Sub mnuFormat_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat.DropDownOpening
        mnuFormat_Image.Enabled = (Not (textControl1.Images.GetItem Is Nothing))
        mnuFormat_TextFrame.Enabled = (Not (textControl1.TextFrames.GetItem Is Nothing))
        mnuFormat_TextFrameMarkerLines.Checked = textControl1.TextFrameMarkerLines
        mnuFormat_DocumentTargetMarkers.Checked = textControl1.DocumentTargetMarkers
    End Sub

    Private Sub mnuFormat_Character_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_Character.Click
        textControl1.FontDialog()
    End Sub

    Private Sub mnuFormat_Paragraph_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_Paragraph.Click
        textControl1.ParagraphFormatDialog()
    End Sub

    Private Sub mnuFormat_Tabs_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_Tabs.Click
        textControl1.TabDialog()
    End Sub

    Private Sub mnuFormat_Styles_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_Styles.Click
        textControl1.FormattingStylesDialog()
    End Sub

    Private Sub mnuFormat_Image_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_Image.Click
        textControl1.ImageAttributesDialog()
    End Sub

    Private Sub mnuFormat_TextFrame_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_TextFrame.Click
        textControl1.TextFrameAttributesDialog()
    End Sub

    Private Sub mnuFormat_TextFrameMarkerLines_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_TextFrameMarkerLines.Click
        textControl1.TextFrameMarkerLines = Not textControl1.TextFrameMarkerLines
    End Sub

    Private Sub mnuFormat_DocumentTargetMarkers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFormat_DocumentTargetMarkers.Click
        textControl1.DocumentTargetMarkers = Not (textControl1.DocumentTargetMarkers)
    End Sub

    Private Sub mnuFormat_TextColor_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_TextColor.Click
        ColorDialog1.ShowDialog()
        textControl1.Selection.ForeColor = ColorDialog1.Color
    End Sub

    Private Sub mnuFormat_TextbackgroundColor_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_TextbackgroundColor.Click
        ColorDialog1.ShowDialog()
        textControl1.Selection.TextBackColor = ColorDialog1.Color
    End Sub

    Private Sub mnuFormat_DocumentBackgroundColor_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_DocumentBackgroundColor.Click
        ColorDialog1.ShowDialog()
        textControl1.BackColor = ColorDialog1.Color
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering.DropDownOpening
        mnuFormat_BulletsAndNumbering_IncreaseLevel.Enabled = (Not (textControl1.Selection.ListFormat.Type = TXTextControl.ListType.None)) AndAlso (textControl1.Selection.ListFormat.Level < TXTextControl.ListFormat.MaxLevel)
        mnuFormat_BulletsAndNumbering_DecreaseLevel.Enabled = (Not (textControl1.Selection.ListFormat.Type = TXTextControl.ListType.None)) AndAlso (textControl1.Selection.ListFormat.Level > 1)
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_Attributes_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_Attributes.Click
        textControl1.ListFormatDialog()
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_IncreaseLevel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_IncreaseLevel.Click
        textControl1.Selection.ListFormat.Level += 1
        textControl1.Selection.ListFormat.LeftIndent += textControl1.Selection.ListFormat.HangingIndent
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_DecreaseLevel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_DecreaseLevel.Click
        textControl1.Selection.ListFormat.Level -= 1
        textControl1.Selection.ListFormat.LeftIndent -= textControl1.Selection.ListFormat.HangingIndent
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_ArabicNumbers_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_ArabicNumbers.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered
        textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.ArabicNumbers
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_CapitalLetters_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_CapitalLetters.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered
        textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.CapitalLetters
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_Letters_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_Letters.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered
        textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.Letters
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_RomanNumbers_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_RomanNumbers.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered
        textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.RomanNumbers
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_SmallRomanNumbers_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_SmallRomanNumbers.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered
        textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.SmallRomanNumbers
    End Sub

    Private Sub mnuFormat_BulletsAndNumbering_Bullets_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFormat_BulletsAndNumbering_Bullets.Click
        textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Bulleted
    End Sub

    Private Sub mnuFormat_HeadersAndFooters_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFormat_HeadersAndFooters.Click
        textControl1.SectionFormatDialog(1)
    End Sub
#End Region

#Region " Insert Menu "

    Private Sub mnuInsert_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert.DropDownOpening
        Try
            mnuInsert_Hyperlink.Enabled = textControl1.HypertextLinks.CanAdd
            mnuInsert_Target.Enabled = textControl1.DocumentTargets.CanAdd
        Catch
        End Try
    End Sub

    Private Sub mnuInsert_File_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert_File.Click
        Try
            FileHandler1.Insert()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuInsert_Image_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert_Image.Click
        Dim NewImage As TXTextControl.Image = New TXTextControl.Image
        textControl1.Images.Add(NewImage, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.ImageInsertionMode.DisplaceText)
    End Sub

    Private Sub mnuInsert_TextFrame_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert_TextFrame.Click
        Dim TextFrameSize As Size = New Size(2000, 1000)
        Dim NewTextFrame As TXTextControl.TextFrame = New TXTextControl.TextFrame(TextFrameSize)
        textControl1.TextFrames.Add(NewTextFrame, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.TextFrameInsertionMode.DisplaceText)

        'Must inform helper of this
        helper.OnTextFrameCreated(NewTextFrame)

    End Sub

    Private Sub mnuInsert_Hyperlink_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert_Hyperlink.Click
        Try
            Dim HyperlinksDialog As frmHyperlink = New frmHyperlink
            HyperlinksDialog.txMain = textControl1
            HyperlinksDialog.ShowDialog()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuInsert_Target_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuInsert_Target.Click
        Try
            Dim TargetName As String = ""
            If InputBox.ShowInputBox("Target Name:", TargetName) Then
                Dim Target As TXTextControl.DocumentTarget = New TXTextControl.DocumentTarget(TargetName)
                textControl1.DocumentTargets.Add(Target)
                FileHandler1.DocumentDirty = True
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuInsert_Section_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuInsert_Section.Click
        Dim InsertSectionDialog As frmInsertBreak = New frmInsertBreak
        InsertSectionDialog.tx = textControl1
        InsertSectionDialog.ShowDialog()
    End Sub
#End Region

#Region " View Menu "

    Private Sub mnuView_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView.DropDownOpening
        mnuView_Normal.Checked = (textControl1.ViewMode = TXTextControl.ViewMode.Normal)
        mnuView_PageLayout.Checked = (textControl1.ViewMode = TXTextControl.ViewMode.PageView)
        mnuView_Toolbar.Checked = toolStrip1.Visible
        mnuView_ButtonBar.Checked = buttonBar1.Visible
        mnuView_StatusBar.Checked = statusBar1.Visible
        mnuView_HorizontalRuler.Checked = rulerBar1.Visible
        mnuView_VerticalRuler.Checked = rulerBar2.Visible
        mnuView_HeadersAndFooters.Enabled = (textControl1.ViewMode = TXTextControl.ViewMode.PageView)
    End Sub

    Private Sub mnuView_Normal_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Normal.Click
        textControl1.ViewMode = TXTextControl.ViewMode.Normal
    End Sub

    Private Sub mnuView_PageLayout_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_PageLayout.Click
        textControl1.ViewMode = TXTextControl.ViewMode.PageView
    End Sub

    Private Sub mnuView_HeadersAndFooters_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_HeadersAndFooters.Click
        Try
            Dim Header As TXTextControl.HeaderFooter
            If textControl1.HeadersAndFooters.Count = 0 Then
                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header)
            End If
            textControl1.HeaderFooterActivationStyle = TXTextControl.HeaderFooterActivationStyle.ActivateClick
            Header = textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header)
            If Not (Header Is Nothing) Then
                Header.Activate()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuView_Toolbar_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Toolbar.Click
        toolStrip1.Visible = Not toolStrip1.Visible
    End Sub

    Private Sub mnuView_ButtonBar_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_ButtonBar.Click
        buttonBar1.Visible = Not buttonBar1.Visible
    End Sub

    Private Sub mnuView_StatusBar_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_StatusBar.Click
        statusBar1.Visible = Not statusBar1.Visible
    End Sub

    Private Sub mnuView_HorizontalRuler_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_HorizontalRuler.Click
        rulerBar1.Visible = Not rulerBar1.Visible
    End Sub

    Private Sub mnuView_VerticalRuler_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_VerticalRuler.Click
        rulerBar2.Visible = Not rulerBar2.Visible
    End Sub

    Private Sub mnuView_Zoom_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom.Click
        mnuView_Zoom_25.Checked = (textControl1.ZoomFactor = 25)
        mnuView_Zoom_50.Checked = (textControl1.ZoomFactor = 50)
        mnuView_Zoom_75.Checked = (textControl1.ZoomFactor = 75)
        mnuView_Zoom_100.Checked = (textControl1.ZoomFactor = 100)
        mnuView_Zoom_150.Checked = (textControl1.ZoomFactor = 150)
        mnuView_Zoom_200.Checked = (textControl1.ZoomFactor = 200)
        mnuView_Zoom_300.Checked = (textControl1.ZoomFactor = 300)
    End Sub

    Private Sub mnuView_Zoom_25_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_25.Click
        textControl1.ZoomFactor = 25
    End Sub

    Private Sub mnuView_Zoom_50_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_50.Click
        textControl1.ZoomFactor = 50
    End Sub

    Private Sub mnuView_Zoom_75_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_75.Click
        textControl1.ZoomFactor = 75
    End Sub

    Private Sub mnuView_Zoom_100_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_100.Click
        textControl1.ZoomFactor = 100
    End Sub

    Private Sub mnuView_Zoom_150_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_150.Click
        textControl1.ZoomFactor = 150
    End Sub

    Private Sub mnuView_Zoom_200_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_200.Click
        textControl1.ZoomFactor = 200
    End Sub

    Private Sub mnuView_Zoom_300_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuView_Zoom_300.Click
        textControl1.ZoomFactor = 300
    End Sub

#End Region

#Region " Edit Menu "

    Private Sub mnuEdit_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit.DropDownOpening
        mnuEdit_Undo.Enabled = textControl1.CanUndo
        mnuEdit_Redo.Enabled = textControl1.CanRedo
        mnuEdit_Cut.Enabled = textControl1.CanCopy
        mnuEdit_Copy.Enabled = textControl1.CanCopy
        mnuEdit_Paste.Enabled = textControl1.CanPaste
        mnuEdit_Undo.Text = "Undo " + textControl1.UndoActionName
        mnuEdit_Redo.Text = "Redo " + textControl1.RedoActionName
        Try
            mnuEdit_Hyperlink.Enabled = (Not (textControl1.HypertextLinks.GetItem Is Nothing)) OrElse (Not (textControl1.DocumentLinks.GetItem Is Nothing))
            mnuEdit_Target.Enabled = (Not (textControl1.DocumentTargets.GetItem Is Nothing))
        Catch
        End Try
    End Sub

    Private Sub mnuEdit_Undo_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Undo.Click
        Undo()
    End Sub

    Private Sub mnuEdit_Redo_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Redo.Click
        Redo()
    End Sub

    Private Sub mnuEdit_Cut_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Cut.Click
        Cut()
    End Sub

    Private Sub mnuEdit_Copy_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Copy.Click
        Copy()
    End Sub

    Private Sub mnuEdit_Paste_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Paste.Click
        Paste()
    End Sub

    Private Sub mnuEdit_Delete_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Delete.Click
        textControl1.Clear()
    End Sub

    Private Sub mnuEdit_SelectAll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_SelectAll.Click
        textControl1.SelectAll()
    End Sub

    Private Sub mnuEdit_Find_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Find.Click
        Find()
    End Sub

    Private Sub mnuEdit_Replace_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Replace.Click
        textControl1.Replace()
    End Sub

    Private Sub mnuEdit_Hyperlink_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Hyperlink.Click
        Try
            Dim HyperlinksDialog As frmHyperlink = New frmHyperlink
            HyperlinksDialog.txMain = textControl1
            HyperlinksDialog.ShowDialog()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Private Sub mnuEdit_Target_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuEdit_Target.Click
        Try
            Dim TargetName As String = ""
            Dim Target As TXTextControl.DocumentTarget = textControl1.DocumentTargets.GetItem
            If Not (Target Is Nothing) Then
                TargetName = Target.Name
                If InputBox.ShowInputBox("Enter a name for this target:", TargetName) Then
                    Target.Name = TargetName
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

#End Region

#Region " TX Text Control Events "

    Private Sub textControl1_ImageRightClicked(ByVal sender As Object, ByVal e As TXTextControl.ImageEventArgs) _
         Handles textControl1.ImageRightClicked
        ContextMenu_Images.Show(Me, Me.PointToClient(Windows.Forms.Cursor.Position))
    End Sub

    Private Sub textControl1_TextFrameRightClicked(ByVal sender As Object, ByVal e As TXTextControl.TextFrameEventArgs) Handles textControl1.TextFrameRightClicked
        'If textControl1.TextFrames.GetItem Is Nothing Then
        '    'ContextMenu_Text.Show(Me, Me.PointToClient(Windows.Forms.Cursor.Position))
        '    'RapidSpell
        '    txTextControlAdapter1.ContextMenuStripDefault = ContextMenu_Text
        'Else
        '    ContextMenu_TextFrames.Show(Me, Me.PointToClient(Windows.Forms.Cursor.Position))
        'End If
    End Sub

    Private Sub textControl1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles textControl1.MouseDown
        'If e.Button = Windows.Forms.MouseButtons.Right Then
        '    'RapidSpell
        '    txTextControlAdapter1.ContextMenuStripDefault = ContextMenu_Text
        'End If
    End Sub

    Private Sub textControl1_Changed(ByVal sender As Object, ByVal e As System.EventArgs) Handles textControl1.Changed
        FileHandler1.DocumentDirty = True
        EnableToolbarButtons()
        UpdateSaveStatus()
    End Sub

    Private Sub textControl1_DocumentLinkClicked(ByVal sender As Object, ByVal e As TXTextControl.DocumentLinkEventArgs) Handles textControl1.DocumentLinkClicked
        e.DocumentLink.ScrollTo()
    End Sub

    Private Sub textControl1_HypertextLinkClicked(ByVal sender As System.Object, ByVal e As TXTextControl.HypertextLinkEventArgs) Handles textControl1.HypertextLinkClicked
        If e.HypertextLink.Target.StartsWith("http") Then
            System.Diagnostics.Process.Start(e.HypertextLink.Target)
        End If
    End Sub

    Private Sub textControl1_InputPositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles textControl1.InputPositionChanged
        EnableToolbarButtons()
    End Sub

    Private Sub textControl1_HeaderFooterActivated(ByVal sender As Object, ByVal e As TXTextControl.HeaderFooterEventArgs) Handles textControl1.HeaderFooterActivated
        m_ActiveHeaderFooter = e.HeaderFooter
        EnableToolbarButtons()
    End Sub

    Private Sub textControl1_HeaderFooterDeactivated(ByVal sender As Object, ByVal e As TXTextControl.HeaderFooterEventArgs) Handles textControl1.HeaderFooterDeactivated
        m_ActiveHeaderFooter = Nothing
        EnableToolbarButtons()
    End Sub

    Private Sub textControl1_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles textControl1.GotFocus
        EnableToolbarButtons()
    End Sub
#End Region

#Region " Menu Functions "

    Public Sub FileSave()
        Try
            FileHandler1.FileSave()
            If Not (FileHandler1.DocumentFileName = "") Then
                Me.Text = FileHandler1.DocumentFileName
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "TX Text Control Words")
        End Try
    End Sub

    Public Sub FileNew()
        Dim Result As System.Windows.Forms.DialogResult

        If FileHandler1.DocumentDirty Then
            Result = MessageBox.Show("Save changes to " + Me.Text + "?", "Question", MessageBoxButtons.YesNoCancel)
            If Result = System.Windows.Forms.DialogResult.Yes Then
                FileHandler1.FileSave()
                If FileHandler1.DocumentFileName = "" Then
                    Return
                End If
            Else
                If Result = System.Windows.Forms.DialogResult.Cancel Then
                    Return
                End If
            End If
        End If
        FileHandler1.DocumentFileName = ""
        textControl1.ResetContents()
        Me.Text = "TX Text Control Words - [untitled]"
        FileHandler1.DocumentDirty = False
        UpdateSaveStatus()
    End Sub

    Public Sub FileOpen()
        Dim Result As System.Windows.Forms.DialogResult

        If FileHandler1.DocumentDirty Then
            Result = MessageBox.Show("Save changes to " + Me.Text + "?", "Question", MessageBoxButtons.YesNoCancel)
            If Result = System.Windows.Forms.DialogResult.Yes Then
                FileHandler1.FileSave()
                If FileHandler1.DocumentFileName = "" Then
                    Return
                End If
            Else
                If Result = System.Windows.Forms.DialogResult.Cancel Then
                    Return
                End If
            End If
        End If
        FileHandler1.DocumentFileName = ""
        If FileHandler1.FileOpen() Then
            Me.Text = FileHandler1.DocumentFileName
        End If

        'RapidSpell-
        helper.OnDocumentOpen()
        '-
    End Sub

    Public Sub Print()
        textControl1.Print("TX Text Control Words Document")
    End Sub

    Public Sub PrintPreview()
        textControl1.PrintPreview("TX Text Control Words Document")
    End Sub

    Public Sub Cut()
        textControl1.Cut()
    End Sub

    Public Sub Copy()
        textControl1.Copy()
    End Sub

    Public Sub Paste()
        textControl1.Paste()
    End Sub

    Public Sub Undo()
        textControl1.Undo()
    End Sub

    Public Sub Redo()
        textControl1.Redo()
    End Sub

    Public Sub Find()
        textControl1.Find()
    End Sub

    Public Sub About()
        Dim applicationCopyright As AssemblyCopyrightAttribute = CType(AssemblyCopyrightAttribute.GetCustomAttribute(System.Reflection.Assembly.GetExecutingAssembly, GetType(AssemblyCopyrightAttribute)), AssemblyCopyrightAttribute)
        MessageBox.Show(Me.ProductName.ToString _
        + Microsoft.VisualBasic.Chr(13) _
        + applicationCopyright.Copyright.ToString _
        + Microsoft.VisualBasic.Chr(13) _
        + Microsoft.VisualBasic.Chr(13) _
        + "Icons by Mark James, www.famfamfam.com", "About TX Text Control Words", MessageBoxButtons.OK)
    End Sub

    Public Sub SwitchBetweenHeaderAndFooter()
        If Not (m_ActiveHeaderFooter Is Nothing) Then
            Try
                Select Case m_ActiveHeaderFooter.Type
                    Case TXTextControl.HeaderFooterType.Footer
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate()
                        End If

                    Case TXTextControl.HeaderFooterType.Header
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Footer)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate()
                        End If

                    Case TXTextControl.HeaderFooterType.FirstPageFooter
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.FirstPageHeader)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate()
                        End If

                    Case TXTextControl.HeaderFooterType.FirstPageHeader
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.FirstPageFooter)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate()
                        End If
                End Select
            Catch ex As Exception

            End Try
        End If
    End Sub

    Public Sub GoToFirstPage()
        If Not (m_ActiveHeaderFooter Is Nothing) Then
            Try
                Select Case m_ActiveHeaderFooter.Type
                    Case TXTextControl.HeaderFooterType.Header
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate()
                        End If
                    Case TXTextControl.HeaderFooterType.Footer
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate()
                        End If
                End Select
            Catch ex As Exception

            End Try
        End If
    End Sub

    Public Sub RemoveHeaderFooter()
        If Not (m_ActiveHeaderFooter Is Nothing) Then
            textControl1.HeadersAndFooters.Remove(m_ActiveHeaderFooter.Type)
        End If
    End Sub

    Public Sub GoToDefault()
        If Not (m_ActiveHeaderFooter Is Nothing) Then
            Try
                Select Case m_ActiveHeaderFooter.Type
                    Case TXTextControl.HeaderFooterType.FirstPageHeader
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate()
                        End If
                    Case TXTextControl.HeaderFooterType.FirstPageFooter
                        If Not (textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer) Is Nothing) Then
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate()
                        Else
                            textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Footer)
                            textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate()
                        End If
                End Select
            Catch ex As Exception

            End Try
        End If
    End Sub

    Function GetNumberOfPages() As Integer
        Dim pages As Integer = 0
        For i As Integer = 1 To textControl1.Pages
            If textControl1.GetPages.Item(i).Section = textControl1.Sections.GetItem().Number Then
                pages += 1
            End If
        Next
        Return pages
    End Function

#End Region

#Region " Toolbar Events "

    Public Sub EnableToolbarButtons()
        toolStrip1.Items(7).Enabled = Me.CanCopy
        toolStrip1.Items(8).Enabled = Me.CanCopy
        toolStrip1.Items(9).Enabled = Me.CanPaste
        toolStrip1.Items(10).Enabled = Me.CanCopy
        toolStrip1.Items(12).Enabled = Me.CanUndo
        toolStrip1.Items(13).Enabled = Me.CanRedo
        If Not (m_ActiveHeaderFooter Is Nothing) Then
            toolStrip1.Items(16).Enabled = True
            toolStrip1.Items(17).Enabled = Not (IsNothing(textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header))) And (GetNumberOfPages() >= 2) And m_ActiveHeaderFooter.Type <> TXTextControl.HeaderFooterType.Header
            toolStrip1.Items(18).Enabled = Not (IsNothing(textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader))) And m_ActiveHeaderFooter.Type <> TXTextControl.HeaderFooterType.FirstPageHeader
            toolStrip1.Items(19).Enabled = True
        Else
            toolStrip1.Items(16).Enabled = False
            toolStrip1.Items(17).Enabled = False
            toolStrip1.Items(18).Enabled = False
            toolStrip1.Items(19).Enabled = False
        End If
    End Sub

    Private Sub toolStripButton1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton1.Click
        FileNew()
    End Sub

    Private Sub toolStripButton2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton2.Click
        FileOpen()
    End Sub

    Private Sub toolStripButton3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton3.Click
        FileSave()
    End Sub

    Private Sub toolStripButton4_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton4.Click
        Print()
    End Sub

    Private Sub toolStripButton5_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton5.Click
        PrintPreview()
    End Sub

    Private Sub toolStripButton6_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton6.Click
        Cut()
    End Sub

    Private Sub toolStripButton7_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton7.Click
        Copy()
    End Sub

    Private Sub toolStripButton8_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton8.Click
        Paste()
    End Sub

    Private Sub toolStripButton9_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton9.Click
        textControl1.Clear()
    End Sub

    Private Sub toolStripButton10_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton10.Click
        Undo()
    End Sub

    Private Sub toolStripButton11_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton11.Click
        Redo()
    End Sub

    Private Sub toolStripButton12_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton12.Click
        Find()
    End Sub

    Private Sub toolStripButton13_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton13.Click
        SwitchBetweenHeaderAndFooter()
    End Sub

    Private Sub ToolStripButton14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButton14.Click
        RemoveHeaderFooter()
    End Sub

    Private Sub toolStripButton15_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton15.Click
        GoToDefault()
    End Sub

    Private Sub toolStripButton16_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton16.Click
        GoToFirstPage()
    End Sub

#End Region

#Region " Context Menus "

    Private Sub ContextMenu_Text_Popup(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles ContextMenu_Text.Opening
        ContextMenu_Text_TableProperties.Enabled = (Not (textControl1.Tables.GetItem Is Nothing))
        ContextMenu_Text_Cut.Enabled = textControl1.CanCopy
        ContextMenu_Text_Copy.Enabled = textControl1.CanCopy
        ContextMenu_Text_Paste.Enabled = textControl1.CanPaste

        Dim suggs As ToolStripItem() = Thesaurus1.GetSuggestionToolStripItems()
        synonymsToolStripMenuItem.DropDown.Items.Clear()
        'If we have suggestions, add them too
        If (Not suggs Is Nothing) Then

            synonymsToolStripMenuItem.DropDown.Items.AddRange(suggs)

        Else

            Dim noSuggs As New ToolStripMenuItem("No suggestions")
            noSuggs.Enabled = False
            synonymsToolStripMenuItem.DropDown.Items.Add(noSuggs)
        End If
    End Sub

    Private Sub ContextMenu_Images_Popup(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles ContextMenu_Images.Opening
        ContextMenu_Images_Cut.Enabled = textControl1.CanCopy
        ContextMenu_Images_Copy.Enabled = textControl1.CanCopy
        ContextMenu_Images_Paste.Enabled = textControl1.CanPaste
    End Sub

    Private Sub ContextMenu_TextFrames_Popup(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles ContextMenu_TextFrames.Opening
        ContextMenu_TextFrames_Cut.Enabled = textControl1.CanCopy
        ContextMenu_TextFrames_Copy.Enabled = textControl1.CanCopy
        ContextMenu_TextFrames_Paste.Enabled = textControl1.CanPaste
    End Sub

    Private Sub ContextMenu_TextFrames_Cut_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_TextFrames_Cut.Click
        Cut()
    End Sub

    Private Sub ContextMenu_TextFrames_Copy_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_TextFrames_Copy.Click
        Copy()
    End Sub

    Private Sub ContextMenu_TextFrames_Paste_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_TextFrames_Paste.Click
        Paste()
    End Sub

    Private Sub ContextMenu_TextFrames_FormatTextFrame_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_TextFrames_FormatTextFrame.Click
        textControl1.TextFrameAttributesDialog()
    End Sub

    Private Sub ContextMenu_Images_Cut_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Images_Cut.Click
        Cut()
    End Sub

    Private Sub ContextMenu_Images_Copy_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Images_Copy.Click
        Copy()
    End Sub

    Private Sub ContextMenu_Images_Paste_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Images_Paste.Click
        Paste()
    End Sub

    Private Sub ContextMenu_Images_FormatImage_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Images_FormatImage.Click
        textControl1.ImageAttributesDialog()
    End Sub

    Private Sub ContextMenu_Text_Cut_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_Cut.Click
        Cut()
    End Sub

    Private Sub ContextMenu_Text_Copy_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_Copy.Click
        Copy()
    End Sub

    Private Sub ContextMenu_Text_Paste_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_Paste.Click
        Paste()
    End Sub

    Private Sub ContextMenu_Text_Character_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_Character.Click
        textControl1.FontDialog()
    End Sub

    Private Sub ContextMenu_Text_Paragraph_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_Paragraph.Click
        textControl1.ParagraphFormatDialog()
    End Sub

    Private Sub ContextMenu_Text_BulletsAndNumbering_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_BulletsAndNumbering.Click
        textControl1.ListFormatDialog()
    End Sub

    Private Sub ContextMenu_Text_InsertTable_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_InsertTable.Click
        Dim InsertTableDialog As frmInsertTable = New frmInsertTable
        InsertTableDialog.tx = textControl1
        InsertTableDialog.ShowDialog()
    End Sub

    Private Sub ContextMenu_Text_TableProperties_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ContextMenu_Text_TableProperties.Click
        textControl1.TableFormatDialog()
    End Sub
#End Region


    Private Sub SpellCheckToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SpellCheckToolStripMenuItem.Click
        RapidSpellDialog1.Check()
    End Sub

    Private Sub ThesaurusToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThesaurusToolStripMenuItem.Click
        If Me.textControl1.Text.Length = 0 Then
            MessageBox.Show("You must select a word to use the thesaurus.")
        Else
            Thesaurus1.DialogForm.TopMost = True
            Me.Thesaurus1.Show()
        End If
    End Sub
End Class
